module Leaderboard exposing (main)

import Bootstrap.Form.Input as Input
import Bootstrap.Form.InputGroup as InputGroup
import Bootstrap.Grid as Grid
import Bootstrap.Grid.Col as Col
import Bootstrap.Grid.Row as Row
import Bootstrap.ListGroup as ListGroup
import Browser
import Date
import Debug exposing (toString)
import Html exposing (..)
import Html.Attributes exposing (..)
import Html.Events exposing (onClick, onInput)
import Http
import Json.Decode as Decode exposing (Decoder)
import Json.Decode.Pipeline exposing (hardcoded, optional, required)
import Task exposing (Task)



--- CONSTANTS


apiUrlPrefix : String
apiUrlPrefix =
    "https://acicchetti.dev/crossword/v1"



--- DECODERS


entryDecoder : Decoder Entry
entryDecoder =
    Decode.succeed Entry
        |> required "name" Decode.string
        |> required "time" Decode.string


scoreDecoder : String -> Decoder Score
scoreDecoder score =
    Decode.string



--- MODEL


type alias Model =
    { date : String
    , entries : List Entry
    , alertMessage : Maybe String
    , newDate : String
    }


type alias Entry =
    { name : String
    , score : Score
    }


type alias Score =
    String



--- UPDATE


type Msg
    = NewDate String
    | NewEntries (Result Http.Error (List Entry))


update : Msg -> Model -> ( Model, Cmd Msg )
update msg model =
    case msg of
        NewDate newDate ->
            ( { model
                | date = newDate
                , entries = []
                , newDate = ""
              }
            , getScores newDate
            )

        NewEntries (Ok newEntries) ->
            ( { model | entries = List.sortBy .score newEntries }, Cmd.none )

        NewEntries (Err err) ->
            ( model, Cmd.none )



--- COMMANDS


getScores : String -> Cmd Msg
getScores newDate =
    Http.request
        { method = "GET"
        , headers = [ Http.header "Access-Control-Allow-Origin" "https://acicchetti.dev" ]
        , url = apiUrlPrefix ++ "/" ++ newDate
        , body = Http.emptyBody
        , expect = Http.expectJson NewEntries (Decode.list entryDecoder)
        , timeout = Nothing
        , tracker = Nothing
        }



--- VIEW


viewEntry : Entry -> ListGroup.Item Msg
viewEntry entry =
    ListGroup.li []
        [ div [] [ text entry.name ]
        , div [] [ text entry.score ]
        ]


viewEntryList : List Entry -> Html Msg
viewEntryList entries =
    div [ class "container-fluid" ]
        [ ListGroup.ul
            (List.map viewEntry entries)
        ]


viewDatePicker : Model -> Grid.Column Msg
viewDatePicker model =
    Grid.col []
        [ InputGroup.config
            (InputGroup.date
                [ Input.placeholder model.date
                , Input.onInput NewDate
                , Input.value model.date
                ]
            )
            |> InputGroup.predecessors [ InputGroup.span [] [ text "Date: " ] ]
            |> InputGroup.view
        ]


view : Model -> Html Msg
view model =
    Grid.containerFluid []
        [ Grid.row [] [ viewDatePicker model ]
        , viewEntryList model.entries
        , text (Maybe.withDefault "" model.alertMessage)
        ]



--- SUBSCRIPTIONS


subscriptions : Model -> Sub Msg
subscriptions model =
    Sub.none


init : () -> ( Model, Cmd Msg )
init _ =
    ( Model "2019-09-29" [] Nothing ""
    , getScores "2019-09-29"
    )


main : Program () Model Msg
main =
    Browser.element
        { init = init
        , view = view
        , update = update
        , subscriptions = subscriptions
        }
